﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2017 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HEAL.Attic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Parameters;

namespace HeuristicLab.Problems.GeneralizedQuadraticAssignment.Algorithms.LocalSearch {
  [Item("Local Search Context", "Context for local search algorithms.")]
  [StorableType("1AD51259-7DDB-43C1-A4A0-9E8B652988B1")]
  public sealed class LocalSearchContext : SingleSolutionContext<ISingleObjectiveSolutionScope<GQAPSolution>> {
    [Storable]
    private IValueParameter<GQAP> problem;
    public GQAP Problem {
      get { return problem.Value; }
      set { problem.Value = value; }
    }

    [Storable]
    private IValueParameter<GQAPSolution> bestSolution;
    public GQAPSolution BestSolution {
      get { return bestSolution.Value; }
      set { bestSolution.Value = value; }
    }
    
    [StorableConstructor]
    private LocalSearchContext(StorableConstructorFlag _) : base(_) { }
    private LocalSearchContext(LocalSearchContext original, Cloner cloner)
    : base(original, cloner) {
      problem = cloner.Clone(original.problem);
      bestSolution = cloner.Clone(original.bestSolution);
    }
    public LocalSearchContext() {
      Parameters.Add(problem = new ValueParameter<GQAP>("Problem"));
      Parameters.Add(bestSolution = new ValueParameter<GQAPSolution>("BestFoundSolution"));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new LocalSearchContext(this, cloner);
    }

    public ISingleObjectiveSolutionScope<GQAPSolution> ToScope(GQAPSolution code, double fitness = double.NaN) {
      var name = Problem.Encoding.Name;
      var scope = new SingleObjectiveSolutionScope<GQAPSolution>(code,
        name + "Solution", fitness, Problem.Evaluator.QualityParameter.ActualName) {
        Parent = Scope
      };
      scope.Variables.Add(new Variable(name, code.Assignment));
      scope.Variables.Add(new Variable("Evaluation", code.Evaluation));
      return scope;
    }
  }
}